"""The credits screen for the game"""

import random
import math
import pygame
import threading

import serge.actor
import serge.visual
import serge.events
import serge.common
import serge.blocks.utils
import serge.blocks.visualblocks
import serge.blocks.behaviours
import serge.blocks.actors

if serge.common.PYMUNK_OK:
    import pymunk
    import serge.physical

from theme import G, theme
import common 
import uielements


### Credits ### 
AUTHOR = 'Paul Paterson'
URL = 'http://perpetualpyramid.com'
MUSIC = []
BUILT_USING = 'pygame'
GAME_ENGINE = 'serge'
FONTS = ['NonFiction']


class CreditsScreen(uielements.StandardUI):
    """The logic for the credits screen"""

    confirm_options = ['Continue', 'Quit']

    def __init__(self, options):
        """Initialise the screen"""
        super(CreditsScreen, self).__init__('item', 'help-screen')
        self.options = options

    def addedToWorld(self, world):
        """The screen was added to the world"""
        super(CreditsScreen, self).addedToWorld(world)
        #
        self.clock_digits.setValue(common.version)
        self.flags_digits.setValue('....')
        #
        # Version
        serge.blocks.utils.addTextToWorld(
            world, 'Version', 'version', theme.getTheme('credits-screen'), 'main'
        )
        #
        self.difficulty_menu.active = False
        self.mode_menu.active = False
        self.mode_lights.setStates(False)
        #
        # Logo
        the_theme = theme.getTheme('credits-screen')
        L = the_theme.getProperty
        serge.blocks.utils.addSpriteActorToWorld(
            world, 'logo', 'logo', 'logo', 'foreground',
            center_position=L('logo-position'))
        #
        # Text on the page
        serge.blocks.utils.addTextItemsToWorld(
            world, [
                ('Game concept, design and coding', 'author-title'),
                (AUTHOR, 'author'),
                (URL, 'url', self.launchURL),
                #('Music', 'music-title1'),
                #('from the FreeMusicArchive.org', 'music-title2'),
                ('Sound', 'sound-title1'),
                ('From FreeSound.org', 'sound-title2'),
                ('Built using', 'built-title'),
                (BUILT_USING, 'built'),
                ('Game engine', 'engine-title'),
                (GAME_ENGINE, 'engine'),
                ('(v' + serge.common.version + ')', 'engine-version'),
                ('Fonts', 'font-title1'),
                ('from CoolText.com', 'font-title2'),
            ],
            the_theme, 'foreground')
        #
        x, y = L('music-position')
        for idx, track in enumerate(MUSIC):
            serge.blocks.utils.addVisualActorToWorld(world, 'title', 'music-item', 
                serge.visual.Text(track, L('music-colour'), 
                font_size=L('music-font-size')), 'foreground', 
                center_position=(x, y+idx*20))
        #
        x, y = L('font-position')
        for idx, font in enumerate(FONTS):
            serge.blocks.utils.addVisualActorToWorld(world, 'title', 'font-item', 
                serge.visual.Text(font, L('font-colour'), 
                font_size=L('font-font-size')), 'foreground', 
                center_position=(x, y+idx*20))
        #
        # Back link
        back = serge.blocks.utils.addVisualActorToWorld(
            world, 'button', 'back',
            serge.visual.Text('Back', L('back-colour'), font_size=L('back-font-size'), font_name='terminal'),
            'foreground',
            center_position=L('back-position'),
            actor_class=serge.blocks.animations.AnimatedActor)
        back.linkEvent(serge.events.E_LEFT_CLICK, serge.blocks.utils.backToPreviousWorld('screen'))
        back.addRegisteredAnimation('pulsed-visibility')
        #
        self.navigation_lights.getButton('submit').setState(True)
        self.navigation_lights.getButton('submit').linkEvent(
            serge.events.E_LEFT_CLICK, serge.blocks.utils.backToPreviousWorld('screen'))

    def updateActor(self, interval, world):
        """Update this actor"""
        super(CreditsScreen, self).updateActor(interval, world)
            
    def launchURL(self, obj, arg):
        """Launch the url"""
        def doit():
            import webbrowser
            webbrowser.open(URL)
        t = threading.Thread(target=doit)
        t.setDaemon(True)
        t.start()

    def menuClick(self, obj, arg):
        """Clicked on the difficulty menu"""


def main(options):
    """Create the main logic"""
    #
    # The screen actor
    s = CreditsScreen(options)
    world = serge.engine.CurrentEngine().getWorld('credits-screen')
    world.addActor(s)
    #
    # The behaviour manager
    manager = serge.blocks.behaviours.BehaviourManager('behaviours', 'behaviours')
    world.addActor(manager)
    #
    # Screenshots
    if options.screenshot:
        manager.assignBehaviour(None, 
            serge.blocks.behaviours.SnapshotOnKey(key=pygame.K_s, size=G('screenshot-size')
                , overwrite=False, location='screenshots'), 'screenshots')

