"""Implements a numeric display that uses sprites for the individual digits"""

import serge.actor
import serge.visual


class SpriteNumeric(serge.actor.Actor):
    """Implements an actor that displays a number using sprites"""

    def __init__(self, tag, name, sprites, characters, fmt=''):
        """Initialise the actor"""
        super(SpriteNumeric, self).__init__(tag, name)
        #
        self.sprites = [serge.visual.Sprites.getItem(sprite) for sprite in sprites]
        self.fmt = fmt
        self.characters = characters
        #
        # Create a visual of the right size - we assume all sprites are the
        # same size
        self.char_width = self.sprites[0].width
        self.char_height = self.sprites[0].height
        self.visual = serge.visual.SurfaceDrawing(self.char_width * self.characters, self.char_height)

    def setValue(self, value):
        """Set the value"""
        #
        # Get the text to use
        if self.fmt:
            text = self.fmt % value
        else:
            text = str(value)
        #
        surface = self.visual.getSurface()
        #
        # Render each digit
        for idx, digit in enumerate(text):
            #
            # Find which item to use
            position = '0123456789. '.find(digit)
            if position < 0:
                raise ValueError('Digit "%s" of %d not recognized' % (digit, value))
            #
            # Render to our surface
            self.sprites[position].renderTo(0, surface, (idx * self.char_width, 0))