"""Represents the squares that can be present on the board"""


class Square(object):
    """Represents the base class of all squares"""

    signal = 0
    trigger = 0
    flagged = False
    revealed = False
    exploded = False
    sprite = 'square'

    def __init__(self, squares, (x, y)):
        """Initialise the square"""
        self.squares = squares
        self.x = x
        self.y = y

    def getNeighbourSquareOffsets(self):
        """Return the neighbours which are active as an offset (dx, dy)"""
        xs, ys = len(self.squares[0]), len(self.squares)
        neighbours = []
        #
        for dx in (-1, 0, 1):
            for dy in (-1, 0, 1):
                cx, cy = self.x + dx, self.y + dy
                if (cx < 0 or cy < 0 or cx >= xs or cy >= ys) or (dx == dy == 0):
                    pass
                else:
                    if self.squares[cy][cx].signal:
                        neighbours.append((dx, dy))
        #
        return neighbours

    def getNeighbours(self):
        """Return the number of neighbour squares which are active"""
        return len(self.getNeighbourSquareOffsets())

    def getSignalStrength(self):
        """Return the signal strength at this location

        This is the number of neighbours + whether we have a signal or not

        """
        return self.getNeighbours() + self.signal * 2 + self.trigger * 2

    def getTriggeredSquares(self):
        """Return the other squares that we would trigger"""
        return []

    def getNumberNeighbourFlags(self):
        """Return the number of flags around us"""
        xs, ys = len(self.squares[0]), len(self.squares)
        flags = 0
        #
        for dx in (-1, 0, 1):
            for dy in (-1, 0, 1):
                cx, cy = self.x + dx, self.y + dy
                if (cx < 0 or cy < 0 or cx >= xs or cy >= ys) or (dx == dy == 0):
                    pass
                else:
                    if self.squares[cy][cx].flagged:
                        flags += 1
        #
        return flags


class Mine(Square):
    """Represents a mined square"""

    signal = 1
    sprite = 'bomb-square'


class Trigger(Square):
    """Represents a triggered square"""

    trigger = 1
    sprite = 'trigger-square'

    def getTriggeredSquares(self):
        """Return the triggered squares"""
        xs, ys = len(self.squares[0]), len(self.squares)
        triggered = []
        #
        for dx in (-1, 0, 1):
            for dy in (-1, 0, 1):
                cx, cy = self.x + dx, self.y + dy
                if (cx < 0 or cy < 0 or cx >= xs or cy >= ys) or (dx == dy == 0):
                    pass
                else:
                    square = self.squares[cy][cx]
                    if not square.revealed and not square.flagged:
                        triggered.append(square)
        #
        return triggered