"""Add some UI elements"""

import pygame

import serge.sound
import serge.blocks.utils
import serge.blocks.actors
import serge.blocks.layout

from theme import G, theme
import spritenumeric
import common
import buttonui
import oscilloscope


class StandardUI(serge.blocks.actors.ScreenActor):
    """The standard UI elements"""

    confirm_options = ['You did', 'not set', 'confirm_options!']

    def addedToWorld(self, world):
        """Added to the world"""
        super(StandardUI, self).addedToWorld(world)
        #
        # Overlay
        serge.blocks.utils.addSpriteActorToWorld(
            world, 'overlay', 'overlay',
            'overlay', layer_name='overlay', center_position=G('overlay-position')
        )
        #
        # Clocks
        self.clock_digits = serge.blocks.utils.addActorToWorld(
            world,
            spritenumeric.SpriteNumeric(
                'digits', 'clock-digits', common.DIGIT_SPRITES, 4),
            layer_name='buttons',
            center_position=G('clock-digits-position'),
        )
        self.flags_digits = serge.blocks.utils.addActorToWorld(
            world,
            spritenumeric.SpriteNumeric(
                'digits', 'flags-digits', common.DIGIT_SPRITES, 4),
            layer_name='buttons',
            center_position=G('flags-digits-position'),
        )
        #
        self.difficulty_menu = serge.blocks.utils.addActorToWorld(
            world,
            serge.blocks.actors.ToggledMenu(
                'menu', 'difficulty', G('difficulties').keys(),
                serge.blocks.layout.HorizontalBar('hbar', 'hbar', G('menu-width')),
                G('menu-default'),
                on_colour=G('menu-on-colour'),
                off_colour=G('menu-off-colour'),
                font_name='terminal',
                font_size=G('menu-font-size'),
                width=G('menu-item-width'),
                height=G('menu-item-height'),
                font_colour=G('menu-font-colour'),
                callback=self.menuClick,
            ),
            layer_name='main',
            center_position=G('menu-position'),
        )
        #
        # The lights for game modes
        self.mode_lights = serge.blocks.utils.addActorToWorld(
            world,
            buttonui.ButtonUI('btn-ui', 'btn-modes', G('game-mode-buttons')),
        )
        self.mode_lights.setLayerName('buttons')
        self.setMode(common.C_GAME_MODE, silent=True)
        self.mode_lights.getButton(common.C_GAME_MODE).setState(True)
        #
        self.mode_menu = serge.blocks.utils.addActorToWorld(
            world,
            serge.blocks.actors.ToggledMenu(
                'menu', 'mode', ['Classic', 'Enhanced'],
                serge.blocks.layout.HorizontalBar('hbar', 'hbar', G('mode-width')),
                G('mode-default'),
                on_colour=G('mode-on-colour'),
                off_colour=G('mode-off-colour'),
                font_name='terminal',
                font_size=G('mode-font-size'),
                width=G('mode-item-width'),
                height=G('mode-item-height'),
                font_colour=G('mode-font-colour'),
                callback=lambda _, option: self.setMode(option),
            ),
            layer_name='main',
            center_position=G('mode-position'),
        )
        #
        # A confirm mode
        self.confirm = serge.blocks.utils.addActorToWorld(
            world,
            serge.blocks.actors.FullScreenMenu(
                'menu', 'confirm', self.confirm_options,
                serge.blocks.layout.VerticalBar(
                    'vbar', 'vbar', height=G('confirm-height')
                ),
                callback=self.confirmClick,
                background_colour=G('confirm-back-colour'),
                font_colour=G('confirm-front-colour'),
                font_size=G('confirm-font-size'),
                font_name='terminal',
            ),
            layer_name='confirm',
        )
        self.confirm.active = False
        #
        # The lights for other functions
        self.navigation_lights = serge.blocks.utils.addActorToWorld(
            world,
            buttonui.ButtonUI('btn-ui', 'btn-navigation', G('navigation-buttons')),
        )
        self.navigation_lights.setLayerName('buttons')
        self.navigation_lights.getButton('on-off-button').linkEvent(serge.events.E_LEFT_CLICK, self.showConfirmMenu)
        #
        # The lights for the difficulty settings
        self.difficulty_lights = serge.blocks.utils.addActorToWorld(
            world,
            buttonui.ButtonUI('btn-ui', 'btn-difficulty', G('hardware-difficulty-buttons')),
        )
        self.difficulty_lights.setLayerName('buttons')

    def menuClick(self, obj, arg):
        """Clicked the difficulty menu"""
        raise NotImplementedError

    def confirmClick(self, option):
        """The confirm menu was clicked"""
        serge.sound.Sounds.play('select')
        self.confirm.active = False
        if option == 'Quit':
            self.engine.stop()

    def showConfirmMenu(self, obj=None, arg=None):
        """Show the confirm menu"""
        serge.sound.Sounds.play('power')
        self.confirm.active = not self.confirm.active

    def updateActor(self, interval, world):
        """Update the actor"""
        super(StandardUI, self).updateActor(interval, world)
        #
        if self.keyboard.isClicked(pygame.K_ESCAPE):
            self.showConfirmMenu()

    def setMode(self, mode, silent=False):
        """Set the game mode"""
        self.log.info('Setting mode to %s' % mode)
        if not silent:
            serge.sound.Sounds.play('button')
        self.mode_lights.setStates(False)
        self.mode_lights.getButton(mode).setState(True)
        common.C_GAME_MODE = mode